#include <stdio.h>
/*
 * Input:
 *	M N
 *	M by N grid in row-major order
 *	(I,J) indicating cell removal selection
 *	(0,0) end of selections
 *	Repeat from M N until either M or N is zero
 *
 * Output:
 *	Game Won	-- if that's the case, or
 *	Remaining grid	-- if the game wasn't won
 */

#define MAXM 10
#define MAXN 40

#define G(r,c) grid[m-r][c-1]	/* map game coords to C subscripts */

char grid[MAXM][MAXN];
char regr[MAXM*MAXN];
char regc[MAXM*MAXN];

int m, n;			/* size of grid */
int nreg;

/* Read the grid in row-major order */
void read_grid()
{
    int i, j;
    int k;
    for (i=1;i<=m;i++)
	for (j=1;j<=n;j++) {
	    scanf("%d",&k);
	    G(i,j) = k;
	}
}

void display()
{
    int i, j;

    for (i=0;i<m;i++) {
	printf("    ");
	for (j=0;j<n;j++)
	    if (grid[i][j] == -1) {
		printf("  ");
		continue;
	    } else printf("%d ", grid[i][j]);
	putchar('\n');
    }
    putchar('\n');
}

/* This is a really "brute-force" algorithm for finding the connectted */
/* region.  We keep a list of the cells found in the region so far,    */
/* initially containing just the cell selected.  Then we repeatedly    */
/* scan the entire grid, looking for a cell that has the same value,   */
/* is adjacent to a cell already in the list, and is itself not in the */
/* list already.  If we find such a cell, we add it to the list,       */
/* increment nreg (the list size), and return true.  When we fail to   */
/* find such a cell, we return false.  Hey, it's cheap - but it works! */

/* Return 1 if the cell at (r,c) is already in the list */
int in_list (int r, int c)
{
    int i;

    for (i=0;i<nreg;i++)
	if (regr[i] == r && regc[i] == c) return 1;
    return 0;
}

/* Return 1 if cell at (r,c) is adjacent to a cell on the list */
int is_adjacent (int r, int c)
{
    int i;

    for (i=0;i<nreg;i++) {
	if ((regr[i]-1 == r || regr[i]+1 == r) && regc[i] == c) return 1;
	if (regr[i] == r && (regc[i]-1 == c || regc[i]+1 == c)) return 1;
    }
    return 0;
}

int find_more(int val)
{
    int r, c;

    for (r=1;r<=m;r++)
	for (c=1;c<=n;c++) {
	    if (G(r,c) != val) continue;
	    if (in_list(r,c)) continue;
	    if (!is_adjacent(r,c)) continue;
	    /* Found one more... */
	    regr[nreg] = r;
	    regc[nreg++] = c;
	    return 1;
	}
    return 0;
}

int find_region(int r, int c)
{
    if (G(r,c) == -1) return 0;		/* nothing in the cell */
    regr[0] = r; regc[0] = c;		/* save it */
    nreg = 1;
    while (find_more(G(r,c))) ;
    return nreg;
}

/* Remove all the cells marked in the region list, then move all cells */
/* above the removed cells down to fill the empty space.  Then move    */
/* entire columns to fill up space vacated by empty columns.           */
void remove_region()
{
    int i, j, k, nm1;

    for (i=0;i<nreg;i++)
	grid[m-regr[i]][regc[i]-1] = -1;
    for (j=0;j<n;j++) {		/* work L to R across columns */
	k = m-1;
	for (i=m-1;i>=0;i--)	/* then bottom to top in each column */
	    if (grid[i][j] != -1) grid[k--][j] = grid[i][j];
	while (k >= 0) grid[k--][j] = -1;
    }

    /* Now fill in the empty columns */
    k = 0;			/* k is next column to be filled */
    for (j=0;j<n;j++) {		/* again, work L to R across columns */
	nm1 = 0;
	for (i=0;i<m;i++) if (grid[i][j] == -1) nm1++; else break;
	if (nm1 != m) {		/* if entire column isn't empty */
	    for (i=0;i<m;i++) grid[i][k] = grid[i][j];
	    k++;
	}
    }
    for (;k<n;k++) for(i=0;i<m;i++) grid[i][k] = -1;
}

main()
{
    int k;
    int nn;
    int ncells;
    int r, c;
    int gridno = 0;

    while(1) {
	scanf("%d%d",&m,&n);
	if (m == 0 || n == 0) exit(0);
	if (m < 1 || m > MAXM || n < 1 || n > MAXN) {
	    printf("Bad parameters.\n");
	    exit(1);
	}
	read_grid();
	ncells = m * n;

	/* The play begins... */

	printf("Grid %d.\n",++gridno);
	for(;;) {
	    scanf("%d%d",&r,&c);
	    if (r == 0 && c == 0) break;	/* end of this grid's data */
	    if (r < 1 || r > m || c < 1 || c > n) continue;
	    nn = find_region(r,c);
	    if (nn < 2) continue;
	    remove_region();
	    ncells -= nn;
	}
	if (ncells == 0) printf("    Game Won\n\n");
	else display();
    }
}
